#!/usr/bin/ruby
#
# Checks checks variables from redis INFO http://redis.io/commands/INFO
#
# ===
#
# Depends on redis gem
# gem install redis
#
# ===
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.
#
# Heavily inspired in check-redis-slave-status.rb
# https://github.com/sensu/sensu-community-plugins/blob/master/plugins/redis/check-redis-slave-status.rb
#

require 'sensu-plugin/check/cli'
require 'redis'
require_relative '../lib/redis_client_options'

class SumoCheckRedisIntValue < Sensu::Plugin::Check::CLI
  include RedisClientOptions

  option :redis_key,
         short: '-K VALUE',
         long: '--redis-key KEY',
         description: 'Redis info key to monitor',
         required: false,
         default: 'role'

  option :redis_compare_value,
         short: '-V VALUE',
         long: '--redis-key-value VALUE',
         description: 'Redis info key value to trigger alarm',
         required: false,
         default: 'master'

   option :comparator,
          short: '-C eq',
          long: '--comparator eq',
          description: 'Operation to compare value (equal eq, greater gt, less lt)',
          required: true,
          default: 'eq'


  def run
    redis = Redis.new(default_redis_options)

    current_value = redis.get(config[:redis_key]).to_i

    if config[:comparator] == "eq"
      if current_value == config[:redis_compare_value].to_i
        ok "Redis #{config[:redis_compare_value]} equals Value stored in redis"
      else
        critical "Redis values are not equal, Stored value is #{current_value}, and check value is: #{config[:redis_compare_value]}"
      end
    elsif config[:comparator] == "gt"
      if current_value > config[:redis_compare_value].to_i
        ok "Redis value stored (#{current_value}) is greater than compaired value (#{config[:redis_compare_value]})"
      else
        critical "Redis value stored (#{current_value}) is not greater than compaired value (#{config[:redis_compare_value]})"
      end
    elsif config[:comparator] == "lt"
      if current_value < config[:redis_compare_value].to_i
        ok "Redis value stored (#{current_value}) is less than compaired value (#{config[:redis_compare_value]})"
      else
        critical "Redis value stored (#{current_value}) is not less than compaired value (#{config[:redis_compare_value]})"
      end
    end

  rescue StandardError
    send(config[:conn_failure_status], "Could not connect to Redis server on #{redis_endpoint}")
  end
end
